AC_DEFUN(NORDUGRID_INIT, [

AC_MSG_CHECKING(for SYSV init scripts)
initddir=
AC_ARG_WITH(sysv-scripts-location,
        [  --sysv-scripts-location=<PATH>     location of the SYSV init scripts (eg. /etc/init.d)],
	[
	initddir=$withval
	],
	[
	for i in /etc/init.d /etc/rc.d/init.d; do
	  if test -d "$i" ; then
	    initddir=$i
	    break
	  fi
        done
        ]
)
if test ! -d "$initddir"; then
  AC_MSG_ERROR(could not find a suitable location for the SYSV init scripts)
fi
AC_MSG_RESULT($initddir)  

monitor_prefix=/var/www/html/gridmonitor

AC_ARG_WITH(monitor-prefix,
        [  --with-monitor-prefix=<PATH>     Specify the location of the Grid Monitor WWW scripts],
	[
	monitor_prefix=$withval
	]
)

monitor_local_prefix=`basename $monitor_prefix`

AC_ARG_WITH(monitor-local-prefix,
        [  --with-monitor-local-prefix=<PATH>     Specify the relative location of the Grid Monitor WWW scripts],
	[
	monitor_local_prefix=$withval
	]
)

AC_MSG_CHECKING(for gSOAP)
AC_ARG_WITH(gsoap-location,
        [  --with-gsoap-location=<PATH>     Specify the gSOAP installation path],
        [
	gsoap_location=$withval
        ],
        [
	gsoap_location=`which soapcpp2 2>/dev/null`
        if test ! "x$gsoap_location" = "x"; then
	  gsoap_location=`echo "$gsoap_location" | sed 's/\/bin\/soapcpp2$//;t;s/.*//'`
	fi
        ]
)
if test ! "x$gsoap_location" = "x"; then
  if test ! -x $gsoap_location/bin/soapcpp2; then
    gsoap_location=
  fi
fi
if test ! "x$gsoap_location" = "x"; then
  AC_MSG_RESULT($gsoap_location)
else
  AC_MSG_RESULT(not found)
fi
AM_CONDITIONAL(HAVE_GSOAP, test ! "x$gsoap_location" = "x")

mysql_config=""
AC_ARG_WITH(mysql-location,
        [  --with-mysql-location=<PATH>     Specify the MySQL installation path],
        [
        AC_PATH_PROG(mysql_config, mysql_config, not found, $with_mysql_location/bin)
        ],
        [
        dnl Search for mysql_config
        AC_PATH_PROG(mysql_config, mysql_config, not found)
        ]
)
AC_MSG_CHECKING(for MySQL)
mysql_location=`echo "$mysql_config" | sed 's/\/bin\/mysql_config$//;t;s/.*//'`
if test "x$mysql_location" = "x"; then
  AC_MSG_RESULT(not found)
else
  AC_MSG_RESULT($mysql_location)
fi
AM_CONDITIONAL(HAVE_MYSQL, test ! "x$mysql_location" = "x")
AM_CONDITIONAL(BUILD_LOGGER, test ! "x$mysql_location" = "x" && test ! "x$gsoap_location" = "x")

AC_MSG_CHECKING(for libxml configuration)
AC_ARG_WITH(xml-config,
        [  --with-xml-config=<PATH>     Specify the path to libxml configuration utility],
        [
	xml_config=$withval
        ],
        [
	xml_config=`which xml-config 2>/dev/null`
        ]
)
if test ! "x$xml_config" = "x"; then
  AC_MSG_RESULT($xml_config)
else
  AC_MSG_RESULT(not found)
fi
AM_CONDITIONAL(HAVE_LIBXML, test ! "x$xml_config" = "x")

prefix=/usr
AC_ARG_WITH(nordugrid-location,
        [  --with-nordugrid-location=<PATH>     Specify the nordugrid installation path],
	[
	prefix=$with_nordugrid_location
	]
)

AC_MSG_CHECKING(for GLOBUS_LOCATION)
AC_ARG_WITH(globus-location,
        [  --with-globus-location=<PATH>     Specify the globus installation path],
        [
	GLOBUS_LOCATION=$withval
	export GLOBUS_LOCATION
        ],
        [
        if test "x$GLOBUS_LOCATION" = "x"; then
          GLOBUS_LOCATION=/opt/globus
          export GLOBUS_LOCATION
        fi
        ]
)
if test ! -d $GLOBUS_LOCATION; then
  AC_MSG_ERROR(could not find GLOBUS_LOCATION)
fi
AC_MSG_RESULT($GLOBUS_LOCATION)  

AC_MSG_CHECKING(for GPT_LOCATION)
AC_ARG_WITH(gpt-location,
        [  --with-gpt-location=<PATH>        Specify the gpt installation path],
        [
        if test "x$withval" = "x"; then
          if test "x$GPT_LOCATION" = "x"; then
            GPT_LOCATION=/opt/gpt
            if test ! -d $GPT_LOCATION; then
              GPT_LOCATION=$GLOBUS_LOCATION
            fi
            export GPT_LOCATION
          fi
        else
          GPT_LOCATION=$withval
          export GPT_LOCATION
        fi
        ],
        [
        if test x$GPT_LOCATION = x; then
          GPT_LOCATION=/opt/gpt
          if test ! -d $GPT_LOCATION; then
            GPT_LOCATION=$GLOBUS_LOCATION
          fi
          export GPT_LOCATION
        fi
        ]
)
AC_MSG_RESULT($GPT_LOCATION)

AC_ARG_WITH(globus-makefile-header,
        [  --with-globus-makefile-header     Use globus-makefile-header to generate makefile stubs],
        if test "x$withval" = "xno" ; then
	  USE_GLOBUS_GPT=.pre
          MAKEFILE_HEADER=:
        else
          AC_CHECK_PROG(MAKEFILE_HEADER,globus-makefile-header,$GLOBUS_LOCATION/bin/globus-makefile-header,not found,$GLOBUS_LOCATION/bin)
          if test "x$MAKEFILE_HEADER" = "xnot found"; then
            AC_CHECK_PROG(MAKEFILE_HEADER,globus-makefile-header,$GLOBUS_LOCATION/sbin/globus-makefile-header,not found,$GLOBUS_LOCATION/sbin)
            if test "x$MAKEFILE_HEADER" = "xnot found"; then
              USE_GLOBUS_GPT=.pre
              MAKEFILE_HEADER=:
            else
              USE_GLOBUS_GPT=
            fi
          else
            USE_GLOBUS_GPT=
          fi
        fi
        ,
        [
        AC_CHECK_PROG(MAKEFILE_HEADER,globus-makefile-header,$GLOBUS_LOCATION/bin/globus-makefile-header,not found,$GLOBUS_LOCATION/bin)
        if test "x$MAKEFILE_HEADER" = "xnot found"; then
          AC_CHECK_PROG(MAKEFILE_HEADER,globus-makefile-header,$GLOBUS_LOCATION/sbin/globus-makefile-header,not found,$GLOBUS_LOCATION/sbin)
          if test "x$MAKEFILE_HEADER" = "xnot found"; then
            USE_GLOBUS_GPT=.pre
            MAKEFILE_HEADER=:
          else
            USE_GLOBUS_GPT=
          fi
        else
          USE_GLOBUS_GPT=
        fi
        ]
) 
dnl Make sure GLOBUS_MAKEFILE_HEADER is substituted
GLOBUS_MAKEFILE_HEADER=${MAKEFILE_HEADER}
AC_SUBST(GLOBUS_MAKEFILE_HEADER)

AC_ARG_WITH(flavor,
	[  --with-flavor=<flavor>     Specify the globus build flavor or without-flavor for a flavor independent  ],

	[
	case $withval in
	no)
		NO_FLAVOR="yes"
		;;
	yes)
		echo "Please specify a globus build flavor" >&2
		exit 1
		;;
	*)
        if test "x$GLOBUS_FLAVOR_NAME" = "xnoflavor"; then
	        echo "Warning: package doesn't build with flavors $withval ignored" >&2
	        echo "Warning: $withval ignored" >&2
        else
		GLOBUS_FLAVOR_NAME=$withval
                if test ! -f "$GLOBUS_LOCATION/etc/globus_core/flavor_$GLOBUS_FLAVOR_NAME.gpt"; then
	                echo "ERROR: Flavor $GLOBUS_FLAVOR_NAME has not been installed" >&2
	                exit 1
                fi

        fi
		;;
	esac
	],
	[ 
        unset GLOBUS_FLAVOR_CONF
        unset GPT_FLAVOR_CONF
        if test "x$GLOBUS_FLAVOR_NAME" = "x"; then
          AC_CHECK_PROG(GLOBUS_FLAVOR_CONF,globus-flavor-configuration,$GLOBUS_LOCATION/sbin/globus-flavor-configuration,not found,$GLOBUS_LOCATION/sbin)

          if test "x$GLOBUS_FLAVOR_CONF" = "xnot found" ; then
            AC_CHECK_PROG(GPT_FLAVOR_CONF,gpt-flavor-configuration,$GPT_LOCATION/sbin/gpt-flavor-configuration,not found,$GPT_LOCATION/sbin)

            if test "x$GPT_FLAVOR_CONF" = "xnot found" ; then
  	      echo "Please specify a globus build flavor" >&2
  	      exit 1
            fi
            FLAVOR_CONF=$GPT_FLAVOR_CONF
          else
            FLAVOR_CONF=$GLOBUS_FLAVOR_CONF
          fi

          GLOBUS_FLAVOR_NAME=`$FLAVOR_CONF |grep -v '^ '|cut -f1 -d: | grep thr |tail -1`

        fi
	]
)
AC_MSG_CHECKING(for Globus flavor)
AC_MSG_RESULT($GLOBUS_FLAVOR_NAME)

AM_CONDITIONAL(WITHOUT_FLAVORS, test "$NO_FLAVOR" = "yes")
AC_SUBST(GLOBUS_FLAVOR_NAME)
AC_CHECK_LIB(globus_ssl_utils_$GLOBUS_FLAVOR_NAME, proxy_cred_desc_new,

AC_MSG_CHECKING(for Globus version)
AC_DEFINE(GLOBUS_2_0)
AC_MSG_RESULT(2.0),

AC_CHECK_LIB(globus_gsi_cert_utils_$GLOBUS_FLAVOR_NAME, globus_gsi_cert_utils_get_x509_name,

AC_MSG_CHECKING(for Globus version)
AC_DEFINE(GLOBUS_2_2)
AC_MSG_RESULT(2.2),

AC_MSG_CHECKING(for Globus version)
AC_MSG_ERROR(could not find the version)
,
-lglobus_gsi_cert_utils_$GLOBUS_FLAVOR_NAME -lglobus_openssl_error_$GLOBUS_FLAVOR_NAME \
-lglobus_openssl_$GLOBUS_FLAVOR_NAME -lssl_$GLOBUS_FLAVOR_NAME -lcrypto_$GLOBUS_FLAVOR_NAME \
-lglobus_common_$GLOBUS_FLAVOR_NAME -L$GLOBUS_LOCATION/lib
),
-lssl_$GLOBUS_FLAVOR_NAME -lcrypto_$GLOBUS_FLAVOR_NAME -L$GLOBUS_LOCATION/lib
)

AC_MSG_CHECKING(for globus_io version)
IO_VERSION=`$GPT_LOCATION/sbin/gpt-query -name globus_io 2>/dev/null | \
grep 'ver:' 2>/dev/null | head -n 1 2>/dev/null | \
sed 's/^.*ver: \([[^ ]]*\).*/\1/' 2>/dev/null`
IO_VERSION_MAJOR=`echo "$IO_VERSION" | sed 's/^\([^.]\).*/\1/'`;
if test "x$IO_VERSION_MAJOR" = "x"; then
  IO_VERSION_MAJOR=0
fi
if test "$IO_VERSION_MAJOR" -lt "4"; then
AC_DEFINE(OLD_GLOBUS_IO)
fi
AC_MSG_RESULT($IO_VERSION)

AC_ARG_ENABLE(experimental,
[  --enable-experimental    Enable experimental code],  
[case "${enableval}" in
  yes) experimental=true ;;
  no)  experimental=false ;;
   *)  AC_MSG_ERROR(bad value ${enableval} for --enable-experimental) ;;
esac],[experimental=false])
experimental_prefix="# "
if test "x$experimental" = "xtrue"; then
dnl  AC_MSG_NOTICE([Turning on experimental code])
  experimental_prefix=""
fi
AC_SUBST(experimental_prefix)
AM_CONDITIONAL(EXPERIMENTAL, test x$experimental = xtrue)

dnl Add $GLOBUS_LOCATION/lib to search for system dynamic libraries.
dnl Is used to avoid rpath of the specified paths
  
AC_ARG_ENABLE(rpath-hack,
[  --enable-rpath-hack    Do not rpath Globus libraries in binaries],
[case "${enableval}" in
  yes) rpath_hack=true ;;
  no)  rpath_hack=falise;;
   *)  AC_MSG_ERROR(bad value ${enableval} for --enable-rpath-hack) ;;
esac],[rpath_hack=true])
if test "x$rpath_hack" = "xtrue"; then
dnl  AC_MSG_NOTICE([Adding GLOBUS_LOCATION/lib to system dynamic library search path])
  cp -p libtool libtool.orig
  cat libtool.orig | sed "s,\(^sys_lib_dlsearch_path_spec=.*\).$,\1 $GLOBUS_LOCATION/lib\"," > libtool
fi

])
