#include "../std.h"
#include <iostream>

#include "../transfer/proxy.h"
#include <globus_gsi_credential.h>

#include "auth.h"


int AuthUser::match_all(const char* line) {
  return AAA_POSITIVE_MATCH;
}

AuthUser::source_t AuthUser::sources[] = {
  { "all", &AuthUser::match_all },
  { "subject", &AuthUser::match_subject },
  { "file", &AuthUser::match_file },
  { "vo", &AuthUser::match_ldap },
  { "voms", &AuthUser::match_voms },
  { NULL, NULL }
};


AuthUser::AuthUser(const char* s,const char* f) {
  if(s) subject=s;
  if(f) filename=f;
  proxy_file_was_created=false;
}

AuthUser::AuthUser(const AuthUser& a) {
  subject=a.subject;
  filename=a.filename;
  proxy_file_was_created=false;
}

void AuthUser::operator=(const AuthUser& a) {
  voms_data.clear();
  subject=a.subject;
  filename=a.filename;
  proxy_file_was_created=false;
}

void AuthUser::set(const char* s,gss_cred_id_t cred) {
  if(s == NULL) { operator=(cred); return; };
  voms_data.clear();
  subject=s;
  filename="";
  char* p = write_proxy(cred);
  if(p) {
    filename=p;
    proxy_file_was_created=true;
  };
}

void AuthUser::operator=(gss_cred_id_t cred) {
  voms_data.clear();
  subject="";
  filename="";
  proxy_file_was_created=false;
  char* s = write_proxy(cred);
  char* name = NULL;
  if(s) {
    globus_gsi_cred_handle_t h;
    if(globus_gsi_cred_handle_init(&h,GLOBUS_NULL) == GLOBUS_SUCCESS) {
      if(globus_gsi_cred_read_proxy(h,s) == GLOBUS_SUCCESS) {
        if(globus_gsi_cred_get_subject_name(h,&name) != GLOBUS_SUCCESS) {
          name=NULL;
        };
      };
    };
  };
  if(!name) {
    if(s) unlink(s);
  } else {
    subject=name;
    filename=s;
    proxy_file_was_created=true;
  };
} 

AuthUser::~AuthUser(void) {
  if(proxy_file_was_created && filename.length()) unlink(filename.c_str());
}

int AuthUser::evaluate(const char* line) {
  // cerr<<"auth: "<<subject<<" to "<<line<<endl;
  bool invert = false;
  bool no_match = false;
  const char* command = "subject";
  int command_len = 7;
  if(subject.length()==0) return AAA_NO_MATCH;
  if(!line) return AAA_NO_MATCH;
  for(;*line;line++) if(!isspace(*line)) break;
  if(*line == 0) return AAA_NO_MATCH;
  if(*line == '#') return AAA_NO_MATCH;
  if(*line == '-') { line++; invert=true; }
  else if(*line == '+') { line++; };
  if(*line == '!') { no_match=true; line++; };
  if((*line != '/') && (*line != '"')) {
    command=line; 
    for(;*line;line++) if(isspace(*line)) break;
    command_len=line-command;
    for(;*line;line++) if(!isspace(*line)) break;
    if(*line == 0) return AAA_NO_MATCH;
  };
  for(source_t* s = sources;s->cmd;s++) {
    if((strncmp(s->cmd,command,command_len) == 0) && 
       (strlen(s->cmd) == command_len)) {
      int res=(this->*(s->func))(line);
      if(res == AAA_FAILURE) return res;
      if(no_match) {
        if(res==AAA_NO_MATCH) { res=AAA_POSITIVE_MATCH; }
        else { res=AAA_NO_MATCH; };
      };
      if(invert) res=-res;
      return res;
    };
  };
  return AAA_FAILURE; 
}

AuthEvaluator::AuthEvaluator(void):name("") {

}

AuthEvaluator::AuthEvaluator(const char* s):name(s) {

}

AuthEvaluator::~AuthEvaluator(void) {

}

void AuthEvaluator::add(const char* line) {
  l.push_back(line);
}

int AuthEvaluator::evaluate(AuthUser &u) const {
  for(list<string>::const_iterator i = l.begin();i!=l.end();++i) {
    int r = u.evaluate(i->c_str());
    if(r != AAA_NO_MATCH) return r;
  };
  return AAA_NO_MATCH;
}

