#ifndef __DATACACHE_H__
#define __DATACACHE_H__

#include <string>

#include "../cache/cache.h"
#include "datacallback.h"

/*
  High level interface to cache operations (same functionality :) )
  and additional functionality to integrate into grid-manager environment.
*/
class DataCache : public DataCallback {
 private:
  /* path to directory with cache info files */
  string cache_path;
  /* path to directory with cache data files.
     usually it is same as cache_path */
  string cache_data_path;
  /* path used to create link in case cache_directory is visible
     under different name during actual usage */
  string cache_link_path;
  /* identifier used to claim files */
  string id;
  /* needed for low-level interface */
  cache_download_handler cdh;
  /* url was provided - active object */
  bool have_url;
  /* url given */
  string cache_url;
  /* file storing content of url */
  string cache_file;
  /* owner:group of cache */
  uid_t cache_uid;
  gid_t cache_gid;
  bool have_creation_time;
  time_t creation_time;
  bool have_expiration_time;
  time_t expiration_time;
 public:
  /* 
    Constructors and destructor
  */
  DataCache(void);
  DataCache(const char* cache_path,const char* cache_data_path,const char* cache_link_path,const char* id,uid_t cache_uid,gid_t cache_gid);
  DataCache(const DataCache &cache);
  ~DataCache(void);
  /*
   Functionality:
     Prepare cache for downloading file. On success file() will return
     path for cache file.
     This function can block for long time if there is another process
     downloading this url.
   Accepts:
    base_url - url to assign.
   Returns:
    available = true - if file is already in cache.
    true - success.
    false - error.
  */
  bool start(const char* base_url,bool &available);
  /*
   Returns:
    path to file holding content of assigned url.
  */
  const string& file(void) const { return cache_file; };
  /*
   Functionality:
    Should be called after file downloaded or failed
   Accepts:
    failure = true if download failed.
   Returns:
    true
  */
  bool stop(bool failure,bool invalidate);
  /*
   Functionality:
    Should be called to create soft-link to cache file. Underlying
    directories will be created.
   Accepts:
    link_path - path for soft-link.
   Returns:
    true - success
    false - error, usually link already exists
  */ 
  bool link(const char* link_path);
  bool link(const char* link_path,uid_t uid,gid_t gid);
  /*
   Functionality:
    Remove some amount of oldest information from cache.
   Accepts:
    size - amount to be removed.
   Returns:
    true - success
    false - error
  */
  bool clean(unsigned long long int size = 1);
  virtual bool cb(unsigned long long int size);
  operator bool(void) { return (cache_path.length() != 0); };
  bool created_available(void) { return have_creation_time; };
  void created(time_t val) { if(!have_creation_time) creation_time=val; have_creation_time=true; };
  void created_force(time_t val) { creation_time=val; have_creation_time=true; };
  time_t created(void) { return creation_time; };
  bool validtill_available(void) { return have_expiration_time; };
  time_t validtill(void) { return expiration_time; };
  void validtill(time_t val) { if(!have_expiration_time) expiration_time=val; have_expiration_time=true; };
  void validtill_force(time_t val) { expiration_time=val; have_expiration_time=true; };
};

#endif
