#ifndef __DATAMOVE_H__
#define __DATAMOVE_H__

#include "../std.h"
#include <string>
#include <list>

#include "datapoint.h"
#include "datacache.h"
#include "../transfer/url_map.h"

/*
  Moves data from source to destination.
*/

class DataMove {
 private:
/*
  DataPoint* source;
  DataPoint* destination;
*/ 
  bool be_verbose;
  bool force_secure;
  bool force_passive;
  string verbose_prefix;
  bool do_retries;
  unsigned long long int default_min_speed;
  time_t default_min_speed_time;
  unsigned long long int default_min_average_speed;
  time_t default_max_inactivity_time;
 public:
  typedef enum {
    success = 0,
    read_acquire_error = 1,
    write_acquire_error = 2,
    read_resolve_error = 3,
    write_resolve_error = 4,
    preregister_error = 5,
    read_start_error = 6,
    write_start_error = 7,
    read_error = 8,
    write_error = 9,
    transfer_error = 10,
    read_stop_error = 11,
    write_stop_error = 12,
    postregister_error = 13,
    cache_error = 14,
    system_error = 15,
    undefined_error = -1
  } result;
  typedef void(*callback)(DataMove*,DataMove::result,void*);
  DataMove(void);
  ~DataMove(void);
  /*
    Initiate transfer from 'source' to 'destination'.
    cache - control caching of downloaded files (destination 
        url is "file://"). 
    map - used to convert source urls. 
    prefix - added to printed information about data transfer speed.
    if 'cb' is not NULL, transfer is done in separate thread and 'cb'
      is called after transfer completes/fails.
    arg - passed to 'cb'.
    *speed* - used to override defaults (see below).
  */
  result Transfer(DataPoint &source,DataPoint &destination,
                     DataCache &cache,const UrlMap &map,
                     callback cb = NULL,void* arg = NULL,
                     const char* prefix = NULL);
  result Transfer(DataPoint &source,DataPoint &destination,
                     DataCache &cache,const UrlMap &map,
                     unsigned long long int min_speed,time_t min_speed_time,
                     unsigned long long int min_average_speed,
                     time_t max_inactivity_time,
                     callback cb = NULL,void* arg = NULL,
                     const char* prefix = NULL);
  /*
    Get/set printing verbose information about data transfer and speed.
    prefix - add this prefix to printed information.
  */
  bool verbose(void);
  void verbose(bool);
  void verbose(const string& prefix);
  /*
    Get/set if this object should handle retries.
  */
  bool retry(void);
  void retry(bool);
  void secure(bool);
  void passive(bool);
  /*
    Set minimal allowed transfer speed (default is 0) to 'min_speed'. If speed 
    drops below for time longer than 'min_speed_time' error is raised.
  */
  void set_default_min_speed(unsigned long long int min_speed,time_t min_speed_time) {
    default_min_speed=min_speed;
    default_min_speed_time=min_speed_time;
  };
  /*
    Same for average speed (averaged over whole time of transfer).
  */
  void set_default_min_average_speed(unsigned long long int min_average_speed) {
    default_min_average_speed=min_average_speed;
  };
  /*
    Set maximal allowed time for waiting for any data.
  */
  void set_default_max_inactivity_time(time_t max_inactivity_time) {
    default_max_inactivity_time=max_inactivity_time;
  };
};

#endif
