/*********************************************************************
 *
 * Authors: Vincenzo Ciaschini - Vincenzo.Ciaschini@cnaf.infn.it 
 *
 * Copyright (c) 2002, 2003 INFN-CNAF on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or
 * http://www.edg.org/license.html
 *
 * Parts of this code may be based upon or even include verbatim pieces,
 * originally written by other people, in which case the original header
 * follows.
 *
 *********************************************************************/

#include <sys/types.h>
#include <dirent.h>

using namespace std;

#include <voms_api.h>
#include "data.h"

extern bool contact(const string &hostname, int port, const string &contact,
		    const string &command, string &buffer, verror_type &error);
extern bool retrieve(X509 *cert, STACK_OF(X509) *chain, recurse_type how, 
		     string &buffer, string &vo, string &file, verror_type &error);
extern bool verify(string message, vomsdata &voms, verror_type &error, string vdir, string cdir);

vomsdata::vomsdata(string voms_dir, string cert_dir) : voms_cert_dir(voms_dir), ca_cert_dir(cert_dir), error(VERR_NONE), workvo(""), extra_data("")
{
  if (voms_cert_dir.empty()) {
    char *v;
    if ( (v = getenv("X509_VOMS_DIR")))
      voms_cert_dir = string(v);
    else 
      voms_cert_dir = "/etc/grid-security/vomsdir";
  }

  if (ca_cert_dir.empty()) {
    char *c;
    if ((c = getenv("X509_CERT_DIR")))
      ca_cert_dir = string(c);
    else
      ca_cert_dir = "/etc/grid-security/certificates";
  }

  DIR *vdir, *cdir;
  vdir = opendir(voms_cert_dir.c_str());
  cdir = opendir(ca_cert_dir.c_str());

  if (!vdir || !cdir)
    error = VERR_DIR;
    
  if (cdir)
    (void)closedir(cdir);
  if (vdir)
    (void)closedir(vdir);
}

bool vomsdata::Contact(string hostname, int port, string servsubject, string command)
{
  string buffer;

  if (contact(hostname, port, servsubject, command, buffer, error))
    return Import(buffer);

  return false;
}

bool vomsdata::Retrieve(X509 *cert, STACK_OF(X509) *chain, recurse_type how)
{
  string buffer;
  string vo;
  string file;

  if (retrieve(cert, chain, how, buffer, vo, file, error)) {
    workvo = vo;
    extra_data = file;
    return Import(buffer);
  }
  return false;
}

bool vomsdata::Import(string buffer)
{
  return verify(buffer, *this, error, voms_cert_dir, ca_cert_dir);
}

bool vomsdata::Export(string &buffer)
{
  string result;
  string temp;

  for (vector<voms>::iterator v=data.begin(); v != data.end(); ++v) {
    int len;
    string d;

    result += string("SIGLEN:") + stringify(v->siglen, temp) +
      "\nSIGNATURE:" + v->signature + "USER:" + v->user + "\nUCA:" + 
      v->userca + "\nSERVER:" + v->server + "\nSCA:" + v->serverca + 
      "\nVO:" + v->voname + "\nURI:" + v->uri + "\nTIME1:" + v->date1 + 
      "\nTIME2:" + v->date2 + "\nDATALEN:";

    switch (v->type) {
    case TYPE_NODATA:
      d = string("NO DATA\n");
      break;
    case TYPE_CUSTOM:
      d = v->custom;
      break;
    case TYPE_STD:
      for (vector<struct data>::iterator dt = v->std.begin(); dt != v->std.end(); ++dt)
	d += string("GROUP: ") + dt->group + "\nROLE: " + dt->role + 
	  "\nCAP: " + dt->cap + "\n";
      break;
    default:
      error = VERR_TYPE;
      return false;
      break;
    }
    len = d.size();

    result += stringify(len, temp) + "\n" + d;
  }

  buffer = result;
  return true;
}

int getMajorVersionNumber(void) {return 1;}
int getMinorVersionNumber(void) {return 1;}
int getPatchVersionNumber(void) {return 1;}
