#include "../../std.h"

#include "util.h"

int GACLsaveSubstituted(GACLacl* acl,struct _GACLnamevalue *subst,const char* filename) {
  int h;
  GACLacl* acl_;
  h = open(filename,O_CREAT | O_EXCL | O_WRONLY,S_IRUSR | S_IWUSR);
  if(h == -1) return 0;
  close(h);
  if(!GACLsaveAcl((char*)filename,acl)) { remove(filename); return 0; };
  acl_ = GACLloadAcl((char*)filename);
  if(acl_ == NULL) { remove(filename); GACLfreeAcl(acl_); return 1; };
  if(!GACLsubstitute(acl_,subst)) { remove(filename); GACLfreeAcl(acl_); return 1; };
  if(!GACLsaveAcl((char*)filename,acl_)) { remove(filename); GACLfreeAcl(acl_); return 1; };
  GACLfreeAcl(acl_);
  return 0; 
}

int GACLsubstitute(GACLacl* acl,struct _GACLnamevalue *subst) {
  GACLentry* entry;
  for(entry=acl->firstentry;entry;entry=entry->next) {
    GACLcred *cred;
    for(cred=entry->firstcred;cred;cred=cred->next) {
      struct _GACLnamevalue *namevalue;
      for(namevalue=cred->firstname;namevalue;namevalue=namevalue->next) {
        if((namevalue->value)[0] == '$') {
          struct _GACLnamevalue* sub;
          for(sub=subst;sub;sub=sub->next) {
            if(strcmp(sub->name,namevalue->value+1) == 0) {
              char* tmp = strdup(sub->name);
              if(tmp) {
                free(namevalue->value);
                namevalue->value=tmp;
              };
              break;
            };
          };
        };
      };
    };
  };
  return 1;
}

extern GACLentry *GACLparseEntry(xmlNodePtr cur);

GACLacl *GACLacquireAcl(char *str)
{
  xmlDocPtr   doc;
  xmlNodePtr  cur;
  GACLacl    *acl;
  GACLentry  *entry;

  doc = xmlParseMemory(str,strlen(str));
  if (doc == NULL) return NULL;

  cur = xmlDocGetRootElement(doc);

  if (xmlStrcmp(cur->name, (const xmlChar *) "gacl"))
    {
      free(doc);
      free(cur);
      return NULL;
    }

  cur = cur->xmlChildrenNode;

  acl = GACLnewAcl();

  while (cur != NULL)
       {
         entry = GACLparseEntry(cur);
         if (entry == NULL)
           {
             GACLfreeAcl(acl);
             xmlFreeDoc(doc);
             return NULL;
           }

         GACLaddEntry(acl, entry);

         cur=cur->next;
       }

  xmlFreeDoc(doc);
  return acl;
}

GACLperm GACLtestFileAclForSubject(const char *filename,const char* subject) {
  const char* name;
  int path_l;
  char* gname;
  GACLacl* acl;
  struct stat st;
  GACLuser* user;
  GACLperm perm;
  GACLcred* cred;
  const char* gacl_name = ".gacl-";

  if(subject[0] == 0) return GACL_PERM_NONE;
  name=strrchr(filename,'/');
  if(name == NULL) { name=filename; } else { name++; };
  if((*name) == 0) gacl_name = ".gacl";
  path_l=name-filename;
  gname=malloc(path_l+6+strlen(name)+1);
  if(gname == NULL) return GACL_PERM_NONE;
  memcpy(gname,filename,path_l); gname[path_l]=0;
  strcat(gname,gacl_name);
  strcat(gname,name);
  if(stat(gname,&st) == 0) {
    if(!S_ISREG(st.st_mode)) { free(gname); return GACL_PERM_NONE; };
    acl=GACLloadAcl(gname);
  } else {
    acl=GACLloadAclForFile((char*)filename);
  };
  free(gname);
  if(acl == NULL) return GACL_PERM_NONE;
  cred=GACLnewCred("person");
  if(cred==NULL) { GACLfreeAcl(acl); return GACL_PERM_NONE; };
  if(!GACLaddToCred(cred,"dn",(char*)subject)) {
    GACLfreeCred(cred); GACLfreeAcl(acl); return GACL_PERM_NONE;
  };
  /* GACLnewUser does not create copy of cred. But GACLfreeUser destoyrs cred */
  user=GACLnewUser(cred);
  if(user == NULL) {
    GACLfreeAcl(acl); GACLfreeCred(cred); return GACL_PERM_NONE;
  };
  perm=GACLtestUserAcl(acl,user);
  GACLfreeAcl(acl);
  GACLfreeUser(user);
  return perm;
}

