#include "../std.h"

#include <string>

#include <globus_gss_assist.h>
#include <globus_io.h>
#include <globus_ftp_control.h>
#include "userspec.h"
#include "../misc/log_time.h"
#include "../misc/inttostring.h"
#include "config.h"

void userspec_t::free(void) {
//  free(spec->subject);
  if(home)  std::free(home);  home=NULL;
  if(name)  std::free(name);  name=NULL;
  if(group) std::free(group); group=NULL;
  Groups.clear();
  // Keep authentication info to preserve proxy (just in case)
}

userspec_t::userspec_t(void):name(NULL),group(NULL),home(NULL),gridmap(false) {
}

userspec_t::~userspec_t(void) {
  userspec_t::free();
}

bool check_gridmap(const char* dn,char** user,const char* mapfile) {
  string globus_gridmap;
  if(mapfile) {
    globus_gridmap=mapfile;
  }
  else {
    char* tmp=getenv("GRIDMAP");
    if((tmp == NULL) || (tmp[0] == 0)) {
      globus_gridmap="/etc/grid-security/grid-mapfile";
    }
    else { globus_gridmap=tmp; };
  };
  ifstream f(globus_gridmap.c_str());
  if(!f.is_open() ) {
    olog<<"Mapfile is missing at "<<globus_gridmap<<endl;
    return false;
  };
  for(;!f.eof();) {
    char buf[512];
    istream_readline(f,buf,sizeof(buf));
    buf[511]=0;
    if(buf[0] != '"') continue;
    char* pos = strchr(buf+1,'"');
    if(pos == NULL) continue;
    (*pos)=0;
    if(strcmp(buf+1,dn) != 0) continue;
    pos++;
    for(;((*pos)!=0) && (((*pos)==' ')||((*pos)=='\t'));pos++);
    if((*pos) != 0) {
      char* poe = pos;
      for(;((*poe)!=0) && ((*poe)!=' ') && ((*poe)!='\t');poe++);
      (*poe)=0;
    };
    if(user) *user=strdup(pos);
    f.close();
    return true;
  };
  f.close();
  return false;
}

int fill_user_spec(userspec_t *spec,globus_ftp_control_auth_info_t *auth,gss_cred_id_t delegated_cred,globus_ftp_control_handle_t *handle) {
  struct passwd pw_;
  struct group gr_;
  struct passwd *pw;
  struct group *gr;
  char buf[BUFSIZ];

  if(auth->auth_gssapi_subject == NULL) return 1;
//  spec->subject=strdup(auth->auth_gssapi_subject);
  if(!check_gridmap(auth->auth_gssapi_subject,&(spec->name))) {
    olog<<"Warning: there is no local mapping for user"<<endl;
    spec->name=strdup("");
//    free(spec->subject);
  } else {
    if(spec->name[0] == 0) {
      olog<<"Warning: there is no local name for user"<<endl;
    } else {
      spec->gridmap=true;
    };
  };
  spec->user.set(auth->auth_gssapi_subject,delegated_cred);
  if((spec->user.proxy() == NULL) || (spec->user.proxy()[0] == 0)) {
    olog<<"No proxy provided"<<endl;
  } else {
    olog<<"Proxy stored at "<<spec->user.proxy()<<endl;
  };
  if(getuid() == 0 && spec->name[0]) {
    olog<<"Mapped to local user: "<<spec->name<<endl;
    getpwnam_r(spec->name,&pw_,buf,BUFSIZ,&pw);
    if(pw == NULL) {
      olog<<"Local user does not exist"<<endl;
//      free(spec->subject); spec->subject=NULL;
      free(spec->name); spec->name=NULL;
      return 1;
    };
  }
  else {
    if(spec->name) free(spec->name); spec->name=NULL;
    getpwuid_r(getuid(),&pw_,buf,BUFSIZ,&pw);
    if(pw == NULL) {
      olog<<"Warning: running user has no name"<<endl;
//      spec->subject=strdup("unknown");
    }
    else {
      spec->name=strdup(pw->pw_name);
      olog<<"Mapped to running user: "<<spec->name<<endl;
    };
  };
  spec->uid=pw->pw_uid;
  spec->gid=pw->pw_gid;
  olog<<"Mapped to local id: "<<spec->uid<<endl;
  olog<<"Mapped to local group id: "<<spec->gid<<endl;
  spec->home=strdup(pw->pw_dir);
  getgrgid_r(spec->gid,&gr_,buf,BUFSIZ,&gr);
  if(gr == NULL) {
    olog<<"Invalid local group"<<endl;
//    if(spec->subject) free(spec->subject); spec->subject=NULL;
    if(spec->name) free(spec->name); spec->name=NULL;
    if(spec->home) free(spec->home); spec->home=NULL;
    return 1;
  };
  spec->group=strdup(gr->gr_name);
  olog<<"Mapped to local group name: "<<spec->group<<endl;
//  spec->delegated_cred=delegated_cred;
/* fill host info here */
  if(globus_io_tcp_get_remote_address(&(handle->cc_handle.io_handle),
         spec->host,&(spec->port)) != GLOBUS_SUCCESS) {
    spec->port=0;
  };
  return 0;
}

string subst_user_spec(string &in,userspec_t *spec) {
  string out = "";
  int i;
  int last;
  last=0;
  for(i=0;i<in.length();i++) {
    if(in[i] == '%') {
      if(i>last) out+=in.substr(last,i-1);
      i++; if(i>=in.length()) {         };
      switch(in[i]) {
        case 'u': {
          char buf[10];
          snprintf(buf,9,"%i",spec->uid); out+=buf; last=i+1; 
        }; break;
        case 'U': { out+=spec->name; last=i+1; }; break;
        case 'g': {
          char buf[10];
          snprintf(buf,9,"%i",spec->gid); out+=buf; last=i+1; 
        }; break;
        case 'G': { out+=spec->group; last=i+1; }; break;
        case 'D': { out+=spec->user.DN(); last=i+1; }; break;
        case 'H': { out+=spec->home; last=i+1; }; break;
        case '%': { out+='%'; last=i+1; }; break;
        default: {
          olog << "Warning: undefined control sequence: %" << in[i] << endl;
        };
      };
    };
  };
  if(i>last) out+=in.substr(last);
  return out;
}
