#include "../std.h"
#include <globus_common.h>
#include <globus_error.h>
#include <globus_ftp_client.h>
#include <globus_gass_copy.h>
#include <globus_object.h>
#include <iostream>
#include "../misc/log_time.h"

#include "check.h"
#include "autodir.h"

#define GLOBUS_ERROR GLOBUS_ERROR_NO_INFO

typedef struct {
  char* dir;
  char* url;
  char* path;
  globus_ftp_client_handle_t f_handle;
  globus_ftp_client_handleattr_t f_attr;
  globus_gass_copy_handle_t* handle;
  globus_gass_copy_callback_t cb;
  char* s_url;
  char* d_url;
  globus_gass_copy_attr_t* s_attr;
  globus_gass_copy_attr_t* d_attr;
  void* arg;
} finfo_t;

typedef finfo_t globus_autodir_handle_t;
void globus_autodir_handle_init(globus_autodir_handle_t *handle) {
  handle->dir=NULL; handle->url=NULL;
  handle->s_url=NULL; handle->d_url=NULL;
};

void globus_autodir_handle_destroy(globus_autodir_handle_t *handle) {
  if(handle->dir) free(handle->dir);
  if(handle->url) free(handle->url);
  if(handle->s_url) free(handle->s_url);
  if(handle->d_url) free(handle->d_url);
  globus_ftp_client_handle_destroy(&(handle->f_handle));
  globus_ftp_client_handleattr_destroy(&(handle->f_attr));
  free(handle);
};

void exists_callback(void* arg,globus_ftp_client_handle_t* handle,globus_object_t* error);
void mkdir_callback(void* arg,globus_ftp_client_handle_t* handle,globus_object_t* error);

char* get_path_start(char* url) {
  char* s;
  s=strchr(url,':'); if(s==NULL) return NULL;
  s++; if((*s) != '/') return NULL;
  s++; if((*s) != '/') return NULL;
  s++; s=strchr(s,'/'); if(s==NULL) return NULL;
  s++; if((*s) == 0) return NULL;
  s--;
  return s;
}

globus_result_t remove_last_dir(char* dir) {
  char* s=dir;
  if((s=strrchr(dir,'/')) == NULL) return GLOBUS_ERROR;
  (*s)=0;
/*  if(s == dir) return GLOBUS_ERROR; */
  return GLOBUS_SUCCESS;
}

globus_result_t add_last_dir(finfo_t* fi) {
  int l = strlen(fi->dir);
  char* s = fi->url+l;
  char* s_e = s;
  if((*s_e) == 0) { return GLOBUS_ERROR; };
  s_e++; if((s_e=strchr(s_e,'/')) == NULL) { return GLOBUS_ERROR; };
  l = s_e - fi->url; memcpy(fi->dir,fi->url,l);
  return GLOBUS_SUCCESS;
}

globus_result_t globus_gass_copy_register_url_to_url_autodir(
    globus_gass_copy_handle_t *a_handle,
    const char* s_url,globus_gass_copy_attr_t *s_attr,
    const char* d_url,globus_gass_copy_attr_t *d_attr,
    globus_gass_copy_callback_t cb,void* arg) {
  globus_result_t res;
  finfo_t *fi;

  fi = (finfo_t*)malloc(sizeof(finfo_t));
  globus_autodir_handle_init(fi);
  res=globus_ftp_client_handleattr_init(&(fi->f_attr));
  if(res != GLOBUS_SUCCESS) return res;
//  globus_ftp_client_handleattr_set_cache_all(&(fi->f_attr),GLOBUS_TRUE);
  globus_ftp_client_handleattr_set_cache_all(&(fi->f_attr),GLOBUS_FALSE);
  res=globus_ftp_client_handle_init(&(fi->f_handle),&(fi->f_attr));
  if(res != GLOBUS_SUCCESS) return res;


  fi->handle=a_handle;
  fi->url=strdup(d_url);
  fi->dir=strdup(d_url);
  fi->path=get_path_start(fi->dir);
  fi->handle=a_handle;
  fi->s_url=strdup(s_url); fi->d_url=strdup(d_url);
  fi->s_attr=s_attr; fi->d_attr=d_attr;
  fi->cb=cb; fi->arg=arg; 
  if(fi->path == NULL) {
    globus_autodir_handle_destroy(fi);
    return GLOBUS_ERROR;
  };
  remove_last_dir(fi->path);
  /* check if we really can handle automatic directory creation */
  if((!strncmp(d_url,"gsiftp://",9)) || (!strncmp(d_url,"ftp://",6))) {
    globus_ftp_client_operationattr_t o_attr;
    globus_ftp_client_operationattr_init(&o_attr);
   // cache all subsequent operations
    globus_ftp_client_handle_flush_url_state(&(fi->f_handle),fi->url);
    res=globus_ftp_client_exists(&(fi->f_handle),fi->dir,&o_attr,&exists_callback,fi);
    globus_ftp_client_operationattr_destroy(&o_attr);
    if(res != GLOBUS_SUCCESS) {
      globus_autodir_handle_destroy(fi);
    };
  }
  else {
    res=globus_gass_copy_register_url_to_url_check(fi->handle,
            fi->s_url,fi->s_attr,fi->d_url,fi->d_attr,fi->cb,fi->arg);
    if(res != GLOBUS_SUCCESS) {
// olog<<"register_url_to_url failed"<<endl;
      (*(fi->cb))(fi->arg,fi->handle,globus_error_get(res));
      globus_autodir_handle_destroy(fi);
    };
  };
  return res;
}

void exists_callback(void* arg,globus_ftp_client_handle_t* handle,globus_object_t* error) {
  globus_result_t res;
  finfo_t *fi = (finfo_t*)arg;
  if(error != GLOBUS_SUCCESS) {
    if(remove_last_dir(fi->path) != GLOBUS_SUCCESS) {
      (*(fi->cb))(fi->arg,fi->handle,error);
      globus_autodir_handle_destroy(fi);
    };
    globus_ftp_client_operationattr_t o_attr;
    globus_ftp_client_operationattr_init(&o_attr);
    res=globus_ftp_client_exists(handle,fi->dir,&o_attr,&exists_callback,fi);
    globus_ftp_client_operationattr_destroy(&o_attr);
    if(res != GLOBUS_SUCCESS) {
// !!!!!! bad solution - better call callback from separate thread ?? maybe not
      (*(fi->cb))(fi->arg,fi->handle,error);
      globus_autodir_handle_destroy(fi);
    };
    return;
  };
  /* exists */
  if(add_last_dir(fi) != GLOBUS_SUCCESS) {
    /* transfer */
// olog<<"Directory exists - Start real transfer"<<endl;
// Close connection here - save open handles :)
    globus_ftp_client_handle_flush_url_state(&(fi->f_handle),fi->url);
    res=globus_gass_copy_register_url_to_url_check(fi->handle,
            fi->s_url,fi->s_attr,fi->d_url,fi->d_attr,fi->cb,fi->arg);
    if(res != GLOBUS_SUCCESS) {
// olog<<"register_url_to_url failed"<<endl;
// !!!!!! bad solution - better call callback from separate thread ?? maybe not
      (*(fi->cb))(fi->arg,fi->handle,globus_error_get(res));
      globus_autodir_handle_destroy(fi);
    };
    return;
  };
  /* create missing dirs */
  globus_ftp_client_operationattr_t o_attr;
  globus_ftp_client_operationattr_init(&o_attr);
odlog(1)<<"Creating directory: "<<fi->dir<<endl;
  res=globus_ftp_client_mkdir(handle,fi->dir,&o_attr,&mkdir_callback,fi);
  globus_ftp_client_operationattr_destroy(&o_attr);
  if(res != GLOBUS_SUCCESS) {
    (*(fi->cb))(fi->arg,fi->handle,GLOBUS_ERROR);
    globus_autodir_handle_destroy(fi);
  };
}

void mkdir_callback(void* arg,globus_ftp_client_handle_t* handle,globus_object_t* error) {
  globus_result_t res;
  finfo_t *fi = (finfo_t*)arg;
  if(error != GLOBUS_SUCCESS) {
    (*(fi->cb))(fi->arg,fi->handle,error);
    globus_autodir_handle_destroy(fi);
    return;
  };
  if(add_last_dir(fi) != GLOBUS_SUCCESS) {
    /* transfer */
// olog<<"Directory created - Start real transfer"<<endl;
// Close connection here - save open handles :)
    globus_ftp_client_handle_flush_url_state(&(fi->f_handle),fi->url);
    res=globus_gass_copy_register_url_to_url_check(fi->handle,
            fi->s_url,fi->s_attr,fi->d_url,fi->d_attr,fi->cb,fi->arg);
    if(res != GLOBUS_SUCCESS) {
// olog<<"register_url_to_url failed"<<endl;
      (*(fi->cb))(fi->arg,fi->handle,globus_error_get(res));
      globus_autodir_handle_destroy(fi);
    };
    return;
  };
  globus_ftp_client_operationattr_t o_attr;
  globus_ftp_client_operationattr_init(&o_attr);
odlog(1)<<"Creating directory: "<<fi->dir<<endl;
  res=globus_ftp_client_mkdir(handle,fi->dir,&o_attr,&mkdir_callback,fi);
  globus_ftp_client_operationattr_destroy(&o_attr);
  if(res != GLOBUS_SUCCESS) {
    (*(fi->cb))(fi->arg,fi->handle,error);
    globus_autodir_handle_destroy(fi);
  };
  return;
}

