#include "../std.h"
#include <globus_common.h>
#include <globus_ftp_client_plugin.h>
#include "ftp_preallocate_plugin.h"


#define GLOBUS_L_FTP_CLIENT_PREALLOCATE_PLUGIN_NAME "globus_ftp_client_preallocate_plugin"

typedef struct {
  char* path;
  unsigned long long int size;
  int size_command;
  int get_command;
  int size_available;
  globus_ftp_client_preallocate_plugin_callback_t cb;
  void* arg;
} globus_l_ftp_client_preallocate_plugin_t;

void globus_l_ftp_client_preallocate_plugin_command(
    globus_ftp_client_plugin_t *                plugin,
    void *                                      plugin_specific,
    globus_ftp_client_handle_t *                handle,
    const char *                                url,
    const char *                                command) {
  globus_l_ftp_client_preallocate_plugin_t *pl =
          (globus_l_ftp_client_preallocate_plugin_t*)plugin_specific;
  if(pl->path == NULL) return;
  if(strncmp(command,"SIZE ",5) == 0) { pl->size_command=1; }
  else { pl->size_command=0; };
  if((strncmp(command,"RETR ",5) == 0) ||
     (strncmp(command,"ERET ",5) == 0)) { pl->get_command=1; }
  else { pl->get_command=0; };
}

void globus_l_ftp_client_preallocate_plugin_response(
    globus_ftp_client_plugin_t *                plugin,
    void *                                      plugin_specific,
    globus_ftp_client_handle_t *                handle,
    const char *                                url,
    globus_object_t *                           error,
    const globus_ftp_control_response_t *       ftp_response) {
  int h;
  struct stat st;
  off_t old_size;
  char buf[4096];
  int l;
  globus_l_ftp_client_preallocate_plugin_t *pl =
          (globus_l_ftp_client_preallocate_plugin_t*)plugin_specific;
  if(error != GLOBUS_SUCCESS) return;
  if(pl->path == NULL) return;
  if(pl->size_command) {
    if(ftp_response->response_class == GLOBUS_FTP_POSITIVE_COMPLETION_REPLY) {
      if(sscanf(ftp_response->response_buffer,"%*i %llu",&(pl->size)) == 1) {
        pl->size_available=1;
      };
    };
  }
  else if(pl->get_command) {
    if(pl->size_available) {
      if(ftp_response->response_class == GLOBUS_FTP_POSITIVE_PRELIMINARY_REPLY){
fprintf(stderr,"setting file %s to size %llu\n",pl->path,pl->size);
        /* because filesytem can skip empty blocks do real write */
        if((h=open(pl->path,O_WRONLY | O_CREAT)) != -1) {
          old_size=lseek(h,0,SEEK_END);
          if(old_size < pl->size) {
            memset(buf,0xFF,sizeof(buf));
            for(;l>0;) {
              old_size=lseek(h,0,SEEK_END);
              l=4096;
              if(l > (pl->size - old_size)) l = pl->size - old_size;
              if(write(h,buf,l) == -1) {
                /* out of space - call provided callback */
                if(pl->cb) {
                  if((pl->cb)(plugin,pl->path,l,pl->arg) != 0) {
                    /* callback failed */
                    lseek(h,0,SEEK_SET); ftruncate(h,pl->size);
                    globus_ftp_client_plugin_abort(handle);
                    break;
                  };
                }
                else { /* no callback provided - return error */
                  lseek(h,0,SEEK_SET); ftruncate(h,pl->size);
                  globus_ftp_client_plugin_abort(handle);
                  break;
                };
              };
            };
          };
        };
        close(h);
      };
    };
    pl->size_available=0;
  };
}

globus_ftp_client_plugin_t* globus_l_ftp_client_preallocate_plugin_copy(
     globus_ftp_client_plugin_t* plugin_template,void* plugin_specific) {
  globus_ftp_client_plugin_t* plugin;
  globus_result_t res;
  char* url;
  globus_l_ftp_client_preallocate_plugin_t *pl =
          (globus_l_ftp_client_preallocate_plugin_t*)plugin_specific;
  plugin=malloc(sizeof(globus_ftp_client_plugin_t));
  if(plugin==NULL) return GLOBUS_NULL;
/*  fprintf(stderr,"+++++ ftp plugin: copy - %x\n",(unsigned int)plugin); */
  url=NULL;
  if(pl->path) {
    if((url=malloc(strlen(pl->path)+6)) == NULL) {
      free(plugin); return GLOBUS_NULL;
    };
    strcpy(url,"file:"); strcat(url,pl->path);
  };
  if((res=globus_ftp_client_preallocate_plugin_init(plugin,url,pl->cb,pl->arg))!=GLOBUS_SUCCESS) {
    free(url); free(plugin); return GLOBUS_NULL;
  };
  free(url);
  return plugin;
}

void globus_l_ftp_client_preallocate_plugin_destroy(
     globus_ftp_client_plugin_t* plugin,void* plugin_specific) {
/*  fprintf(stderr,"+++++ ftp plugin: destroy - %x\n",(unsigned int)plugin); */
  globus_ftp_client_preallocate_plugin_destroy(plugin);
  free(plugin);
}

void globus_l_ftp_client_preallocate_plugin_get(
    globus_ftp_client_plugin_t *                plugin,
    void *                                      plugin_specific,
    globus_ftp_client_handle_t *                handle,
    const char *                                url,
    const globus_ftp_client_operationattr_t *   attr,
    globus_bool_t                               restart) {
};

globus_result_t globus_ftp_client_preallocate_plugin_init(globus_ftp_client_plugin_t *plugin,const char* url,globus_ftp_client_preallocate_plugin_callback_t cb,void *arg) {
  globus_l_ftp_client_preallocate_plugin_t* pl;
  globus_result_t res;
  const char* p;

  pl=malloc(sizeof(globus_l_ftp_client_preallocate_plugin_t));
  if(pl == NULL) return GLOBUS_ERROR_NO_INFO;
  p=NULL;
  pl->path=NULL;
  if(url) if(strncmp(url,"file:/",6)==0) {
    p=url+6; 
    if((*p)=='/') {
      p++; 
      if((*p)=='/') { p++; }
      else {
        p=strchr(p,'/');
        if(p!=NULL) p++;
      };        
    };
  }; 
  if(p!=NULL) {
    p--;
    pl->path=strdup(p);
    if(pl->path == NULL) { free(pl); return GLOBUS_ERROR_NO_INFO; };
    pl->size_command=0;
    pl->size_available=0;
    pl->get_command=0;
    pl->size=0;
    pl->cb=cb;
    pl->arg=arg;
  };
  if((res=globus_ftp_client_plugin_init(plugin,
       GLOBUS_L_FTP_CLIENT_PREALLOCATE_PLUGIN_NAME,
       GLOBUS_FTP_CLIENT_CMD_MASK_INFORMATION | 
       GLOBUS_FTP_CLIENT_CMD_MASK_FILE_ACTIONS,
       pl)
    ) != GLOBUS_SUCCESS) {
    free(pl); return res;
  };
  if((res=globus_ftp_client_plugin_set_copy_func(plugin,
       &globus_l_ftp_client_preallocate_plugin_copy)) != GLOBUS_SUCCESS) {
    free(pl); return res;
  };
  if((res=globus_ftp_client_plugin_set_destroy_func(plugin,
       &globus_l_ftp_client_preallocate_plugin_destroy)) != GLOBUS_SUCCESS) {
    free(pl); return res;
  };
  if((res=globus_ftp_client_plugin_set_command_func(plugin,
       &globus_l_ftp_client_preallocate_plugin_command)) != GLOBUS_SUCCESS) {
    free(pl); return res;
  };
  if((res=globus_ftp_client_plugin_set_response_func(plugin,
       &globus_l_ftp_client_preallocate_plugin_response)) != GLOBUS_SUCCESS) {
    free(pl); return res;
  };
  if((res=globus_ftp_client_plugin_set_get_func(plugin,
       &globus_l_ftp_client_preallocate_plugin_get)) != GLOBUS_SUCCESS) {
    free(pl); return res;
  };
  return GLOBUS_SUCCESS;
}

globus_result_t globus_ftp_client_preallocate_plugin_destroy(globus_ftp_client_plugin_t *plugin) {
  globus_l_ftp_client_preallocate_plugin_t* pl;
  globus_result_t res;
  if((res=globus_ftp_client_plugin_get_plugin_specific(plugin,(void **)&pl))
                     != GLOBUS_SUCCESS) {
    return res;
  };
  free(pl);
  return globus_ftp_client_plugin_destroy(plugin);
}
