#ifndef GM_REPLICA_H
#define GM_REPLICA_H

#include <string>
#include <list>
#include <globus_replica_catalog.h>
#include <pthread.h>

class RCManager;

class RCLocation {
 public:
  string name;
  string url;
  RCLocation(const char* url,const char* name);
  RCLocation(const string &url,const string &name);
};

class RCFile {
 friend class RCManager;
 private:
  typedef unsigned long long int ullint;
  string name;
  string path;  /* real full name */
  ullint size_i;
  string size;
  bool   size_valid;
  ullint checksum_i;
  string checksum;
  bool   checksum_valid;
  ullint timestamp_i;
  string timestamp;
  bool   timestamp_valid;
 public:
  RCFile(void);
  RCFile(const string &name,const string &path);
  RCFile(const string &name,ullint size,ullint checksum,ullint timestamp);
  RCFile(const string &name_,ullint size_,bool size_valid_,ullint checksum_,
         bool checksum_valid_,ullint timestamp_,bool timestamp_valid_);
  ~RCFile(void);
  bool is_valid(void) { return (name.length() != 0); };
  RCFile& operator= (const RCFile &file);
  ullint get_checksum(void) const { return checksum_i; };
  ullint get_size(void) const { return size_i; };
  ullint get_timestamp(void) const { return timestamp_i; };
};

class RCManager {
 private:
  static pthread_mutex_t sasl_mutex;
  globus_replica_catalog_collection_handle_t handle;
  globus_replica_catalog_collection_handleattr_t attrs;
  globus_replica_catalog_authentication_mode_t auth_mode;
  bool inited;  
 public:
  RCManager(const string &url,const string &manager,const string &pwd,bool secure=false);
  ~RCManager(void);
  bool AddFile(const RCFile &RCFile,bool unique = true);
  bool UpdateFile(const RCFile &RCFile);
  bool AddFileLocation(const RCFile &file,
                const string &host,const string &root_dir,bool unique);
//  bool CheckLocation(const string &host);
  /* from given hostname construct url for sending file to */
  bool GetLocations(const string &host,list<RCLocation> &locs,const char* file_name = NULL);
  bool GetFile(RCFile &file);
  bool RemoveFile(const RCFile &file);
  bool RemoveFileLocation(const RCFile &file,const string &host);
  bool is_open(void) { return inited; };
};

#endif
