#include "std.h"

#include <iostream>
#include <time.h>

#if GLOBUS_2_0
#include <sys/stat.h>
#include <sslutils.h>
#include <openssl/x509.h>
#elif GLOBUS_2_2
#include <globus_gsi_proxy.h>
#include <globus_gsi_system_config.h>
#endif

#include <ldap.h>

#include "CertInfo.h"


#if GLOBUS_2_0

CertInfo::CertInfo (const char * proxy) {

  char * proxy_file = NULL;
  struct stat stx;

  good = false;

  proxy_cred_desc * pcd = proxy_cred_desc_new();
  if (!pcd) {
    cerr << "Could not create proxy credential descriptor." << endl;
    goto exit;
  }

  if (proxy) {
    proxy_file = strdup (proxy);
  }
  else {
    proxy_get_filenames (pcd, 1, NULL, NULL, &proxy_file, NULL, NULL);
    if (!proxy_file) {
      cerr << "Unable to determine proxy file name." << endl;
      goto exit;
    }
  }

  if (stat (proxy_file, &stx) != 0) {
    cerr << "File " << proxy_file << "not found." << endl;
    goto exit;
  }

  pcd->type = CRED_TYPE_PROXY;

  if (proxy_load_user_cert (pcd, proxy_file, NULL, NULL)) {
    cerr << "Unable to load proxy." << endl;
    goto exit;
  }

  if ((pcd->upkey = X509_get_pubkey (pcd->ucert)) == NULL) {
    cerr << "Unable to load public key from proxy." << endl;
    goto exit;
  }

  good = true;

  sn = X509_NAME_oneline (X509_get_issuer_name (pcd->ucert), NULL, 0);
  expires = ASN1_UTCTIME_mktime (X509_get_notAfter (pcd->ucert));

 exit:
  if (pcd) proxy_cred_desc_free (pcd);
  if (proxy_file) free (proxy_file);
}

#elif GLOBUS_2_2

CertInfo::CertInfo (const char * proxy) {

  globus_result_t result;
  char * proxy_filename = NULL;
  globus_gsi_cred_handle_t proxy_cred = NULL;
  X509 * proxy_cert = NULL;
  EVP_PKEY * proxy_pubkey = NULL;
  char * subject = NULL;
  time_t goodtill;

  good = false;

  if (proxy) {
    proxy_filename = strdup (proxy);
  }
  else {
    result = GLOBUS_GSI_SYSCONFIG_GET_PROXY_FILENAME (&proxy_filename,
						      GLOBUS_PROXY_FILE_INPUT);
    if (result != GLOBUS_SUCCESS) {
      cerr << "Couldn't find a valid proxy." << endl;
      goto exit;
    }
  }

  result = globus_gsi_cred_handle_init (&proxy_cred, NULL);
  if (result != GLOBUS_SUCCESS) {
    cerr << "Couldn't initialize proxy credential handle." << endl;
    goto exit;
  }

  result = globus_gsi_cred_read_proxy (proxy_cred, proxy_filename);
  if (result != GLOBUS_SUCCESS) {
    cerr << "Couldn't read proxy from " << proxy_filename << '.' << endl;
    goto exit;
  }

  result = globus_gsi_cred_get_cert (proxy_cred, &proxy_cert);
  if (result != GLOBUS_SUCCESS) {
    cerr << "Couldn't get the proxy certificate from the proxy credential."
	 << endl;
    goto exit;
  }

  if ((proxy_pubkey = X509_get_pubkey(proxy_cert)) == NULL) {
    cerr << "Unable to load public key from proxy." << endl;
    goto exit;
  }

  result = globus_gsi_cred_get_issuer_name (proxy_cred, &subject);
  if (result != GLOBUS_SUCCESS) {
    cerr << "Couldn't get a valid issuer name from the proxy credential."
	 << endl;
    goto exit;
  }

  result = globus_gsi_cred_get_goodtill (proxy_cred, &goodtill);
  if (result != GLOBUS_SUCCESS) {
    cerr << "Couldn't get a valid lifetime for the proxy credential." << endl;
    goto exit;
  }

  sn = subject;
  expires = goodtill;
  good = true;

 exit:
  if (proxy_cred) globus_gsi_cred_handle_destroy (proxy_cred);
  if (proxy_filename) free (proxy_filename);
  if (subject) free (subject);
}

#endif


string CertInfo::GetSN () const { return sn; }

string CertInfo::GetDN () const { return ldap_dcedn2dn(sn.c_str()); }


time_t CertInfo::TimeLeft () const {

  time_t now = time (NULL);
  time_t timeleft = expires - now;
  return (timeleft > 0) ? timeleft : 0;
}


bool CertInfo::operator! () const {

  if (!good) return true;

  if (TimeLeft() == 0) {
    cerr << "Your proxy has expired" << endl;
    cerr << "Please rerun grid-proxy-init" << endl;
    return true;
  }

  if (TimeLeft() < 600) {
    cerr << "Your proxy will soon expire" << endl;
    cerr << "Please rerun grid-proxy-init" << endl;
    return true;
  }

  return false;
}
